@@ -12,7 +12,7 @@ use Math::Round;
 use List::Util qw[ min max reduce shuffle];
 use List::MoreUtils qw(uniq);
 use List::AllUtils qw(sum);
-use Sim::OPT::Stats qw(:all);
+use Sim::OPT::Stats qw( :all );
 use Storable qw(store retrieve lock_store lock_nstore lock_retrieve dclone);
 use IO::Tee;
 use File::Copy qw( move copy );
@@ -37,6 +37,7 @@ use Sim::OPT::Descend;
 use Sim::OPT::Takechance;
 use Sim::OPT::Interlinear;
 use Sim::OPT::Parcoord3d;
+use Sim::OPT::Stats;
 eval { use Sim::OPTcue; 1 };
 eval { use Sim::OPTcue::Patternsearch; 1 };
 eval { use Sim::OPTcue::NelderMead; 1 };
@@ -46,6 +47,7 @@ eval { use Sim::OPTcue::ParticleSwarm; 1 };
 eval { use Sim::OPTcue::SimulatedAnnealing; 1 };
 eval { use Sim::OPTcue::NSGAIII; 1 };
 eval { use Sim::OPTcue::MOEAD; 1 };
+eval { use Sim::OPTcue::SPEA2; 1 };
 
 our @ISA = qw( Exporter );
 #%EXPORT_TAGS = ( DEFAULT => [qw( &opt &prepare )]); # our %EXPORT_TAGS = ( 'all' => [ qw( ) ] );
@@ -70,7 +72,7 @@ $target %dowhat readsweeps $max_processes $computype $calcprocedure %specularrat
 toil genstar solvestar integratebox filterbox__ clean %dowhat @weighttransforms
 );
 
-$VERSION = '0.851';
+$VERSION = '0.857';
 $ABSTRACT = 'Sim::OPT is an optimization and parametric exploration program oriented toward problem decomposition. It can be used with simulation programs receiving text files as input and emitting text files as output. It allows a free mix of sequential and parallel block coordinate searches, as well of searches more complely structured in graphs.';
 
 #################################################################################
@@ -78,6 +80,19 @@ $ABSTRACT = 'Sim::OPT is an optimization and parametric exploration program orie
 #################################################################################
 
 
+BEGIN {
+    package IO::Tee;
+    sub new {
+        shift; # remove class name
+        my $fh = shift; # take the first handle (usually STDOUT)
+        return $fh;
+    }
+    sub FILENO { return 1 }
+    $INC{'IO/Tee.pm'} = __FILE__; # Trick Perl into thinking it is loaded
+}
+
+
+
 our %cryptc;
 our $globcount = 0;   # global counter, starts at 0
 
@@ -2517,11 +2532,24 @@ sub deffiles # IT DEFINED THE FILES TO BE PROCESSED
 	}
 
 	my ( @bux, @buux );
-	if ( ( $dirfiles{starsign} eq "y" ) or
-			( ( $dirfiles{random} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) or
-			( ( $dirfiles{latinhypercube} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) or
-			( ( $dirfiles{factorial} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) or
-			( ( $dirfiles{facecentered} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) )
+	if ( 
+    ( $dirfiles{starsign} eq "y" ) 
+			or ( ( $dirfiles{random} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) 
+			or ( ( $dirfiles{latinhypercube} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) 
+			or ( ( $dirfiles{factorial} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) 
+      or ( ( $dirfiles{facecentered} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) 
+			or ( ( $dirfiles{randompick} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{newrandompick} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{patternsearch} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{neldermead} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{armijo} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{NSGAII} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{NSGAIII} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{pso} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{simulatedannealing} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{MOEAD} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{SPEA2} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) 
+    )
 	{
 		if ( $dirfiles{starsign} eq "y" )
 		{
@@ -2538,10 +2566,24 @@ sub deffiles # IT DEFINED THE FILES TO BE PROCESSED
 
 		my ( $tmpblankfile, $bit );
 		my ( @bag, @fills, @fulls );
-		if ( ( ( $dirfiles{random} eq "y" ) and ( $dowhat{metamodel} eq "ys" ) )
+		if ( 
+      ( ( $dirfiles{random} eq "y" ) and ( $dowhat{metamodel} eq "ys" ) )
 		 	or ( ( $dirfiles{latinhypercube} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
 			or ( ( $dirfiles{factorial} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
-			or ( ( $dirfiles{facecentered} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )	)
+			or ( ( $dirfiles{facecentered} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )	
+      or ( ( $dirfiles{factorial} eq "y" ) and ( $dowhat{metamodel} eq "y" ) ) 
+      or ( ( $dirfiles{randompick} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{newrandompick} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{patternsearch} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{neldermead} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{armijo} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{NSGAII} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{NSGAIII} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{pso} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{simulatedannealing} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{MOEAD} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+      or ( ( $dirfiles{SPEA2} eq "y" ) and ( $dowhat{metamodel} eq "y" ) )
+    )
 		{
 			$tmpblankfile = "$mypath/$file" . "_tmp_gen_blank.csv";
 			$bit = $file . "_";
