@@ -22,6 +22,7 @@ use Sim::OPT::Sim;
 use Sim::OPT::Report;
 use Sim::OPT::Takechance;
 use Sim::OPT::Interlinear;
+use Sim::OPT::Stats;
 eval { use Sim::OPTcue; 1 };
 eval { use Sim::OPTcue::Patternsearch; 1 };
 
@@ -39,7 +40,7 @@ no warnings;
 
 our @EXPORT = qw( descend prepareblank tee ); # our @EXPORT = qw( );
 
-$VERSION = '0.173'; # our $VERSION = '';
+$VERSION = '0.179'; # our $VERSION = '';
 $ABSTRACT = 'Sim::OPT::Descent is an module collaborating with the Sim::OPT module for performing block coordinate descent.';
 
 #########################################################################################
@@ -1358,280 +1359,322 @@ sub descend
 
 #################################################################################
 
+# ------------------------------------------------------------
+# metamodel() dispatcher
+# - default (open): DWGN/Interlinear implementation
+# - optional (closed contract): delegated to Sim::OPTcue when enabled
+# ------------------------------------------------------------
+sub metamodel
+{
+  my ( $dowhat_r, $sortmixed, $file, $dirfiles_r, $blockelts_r, $carrier_r, $metafile,
+    $direction, $starorder, $ordmeta, $varnums_r, $countblock ) = @_;
+
+  my %dowhat = %{ $dowhat_r || {} };
+
+  return unless ( defined($dowhat{metamodel}) && $dowhat{metamodel} eq "y" );
+
+  my $contract = "";
+  $contract = $dowhat{metamodelcontract} if defined $dowhat{metamodelcontract};
+  $contract = $dowhat{metamodel_contract} if ( $contract eq "" && defined $dowhat{metamodel_contract} );
+
+  if ( defined($contract) && $contract eq "cue" )
+  {
+    require Sim::OPTcue;
+    return Sim::OPTcue::metamodel_contract(
+      $dowhat_r, $sortmixed, $file, $dirfiles_r, $blockelts_r, $carrier_r, $metafile,
+      $direction, $starorder, $ordmeta, $varnums_r, $countblock
+    );
+  }
+
+  return _metamodel_dwgn_impl(
+    $dowhat_r, $sortmixed, $file, $dirfiles_r, $blockelts_r, $carrier_r, $metafile,
+    $direction, $starorder, $ordmeta, $varnums_r, $countblock
+  );
+}
 
 
-  sub metamodel
-	{
+
+
+  sub _metamodel_dwgn_impl
+  {
     my ( $dowhat_r, $sortmixed, $file, $dirfiles_r, $blockelts_r, $carrier_r, $metafile,
       $direction, $starorder, $ordmeta, $varnums_r, $countblock ) = @_;
-    my %dowhat = %{ $dowhat_r };
 
-    if ( $dowhat{preprep} ne "" )
-    {
-      $sortmixed = $dowhat{preprep};
-      unless ( -e $sortmixed )
-      {
-        die;
-      }
-    }
-
-    if ( $dowhat{preprep} ne "" )
-    {
-      $sortmixed = $dowhat{preprep};
-      unless ( -e $sortmixed )
-      {
-        die;
-      }
-    }
-
-    my %dirfiles = %{ $dirfiles_r };
+    my %dowhat    = %{ $dowhat_r };
+    my %dirfiles  = %{ $dirfiles_r };
     my @blockelts = @{ $blockelts_r };
-    my %carrier = %{ $carrier_r };
-    my %varnums = %{ $varnums_r };
-    close SORTMIXED;
+    my %carrier   = %{ $carrier_r };
+    my %varnums   = %{ $varnums_r };
 
-		if ( $dowhat{metamodel} eq "y" )
-		{
-		  sub cleanres
-		  {
-		    my ( $sortmixed ) = @_;
-		    open( ORDRES, "$sortmixed") or die;
-		    my $cleanordres = $sortmixed . "_tmp_cleaned.csv";
-		    my @lines = <ORDRES>;
-	      close ORDRES;
+    return unless ( $dowhat{metamodel} eq "y" );
 
-	      open( CLEANORDRES, ">>$cleanordres" ) or die;
-	      foreach my $line ( @lines )
-	      {
-	        $line =~ s/^$mypath\/$file// ;
-	        $line =~ s/^_// ;
-	        print CLEANORDRES $line;
-	      }
-	      close CLEANORDRES;
-	      return( $cleanordres );
-	    }
+    # Optional override of the entrance file (for debugging / staging)
+    if ( defined( $dowhat{preprep} ) and ( $dowhat{preprep} ne "" ) )
+    {
+      $sortmixed = $dowhat{preprep};
+      die "PREPREP file not found: $sortmixed\n" unless ( -e $sortmixed );
+    }
 
+    # metamodel() is outside descend(), so do not rely on descend() lexicals
+    my $mypath = $main::mypath;
+    my $tofile = $main::tofile;
 
-      sub prepareblank
-	    {
-	      my ( $varnums_r, $blankfile, $blockelts_r, $bag_r, $file, $carrier_r ) = @_;
-	      my %varnums = %{ $varnums_r };
-	      my @blockelts = ( sort { $a <=> $b } @{ $blockelts_r } );
-        my @bag = @{ $bag_r };
-        my %carrier = %{ $carrier_r };
-        open( BLANKFILE, ">$blankfile" ) or die;
+    # Interlinear config path (same intent as in descend())
+    my $confinterlinear = $dowhat{confinterlinear};
+    if ( defined( $confinterlinear ) and ( $confinterlinear ne "" ) and defined( $mypath ) and ( $mypath ne "" ) )
+    {
+      $confinterlinear = "$mypath/$confinterlinear";
+    }
 
-        my @box = @{ Sim::OPT::toil( \@blockelts, \%varnums, $bag_r ) };
-	      @box = @{ $box[-1] };
-	      my $integrated_r = Sim::OPT::integratebox( \@box, \%carrier, $file, \@blockelts );
+    # ---------- helpers as closures (capture lexicals safely) ----------
 
-	      my @finalbox = @{ $integrated_r };
+    my $is_num = sub
+    {
+      my ( $v ) = @_;
+      return 0 unless defined $v;
+      return ( $v =~ /^\s*[-+]?\d*\.?\d+(?:[eE][-+]?\d+)?\s*$/ ) ? 1 : 0;
+    };
 
-        foreach my $el ( @finalbox )
-	      {
-	        say BLANKFILE $el->[0];
-	      }
-	      close BLANKFILE;
-        return( \@finalbox )
-	    }
+    my $canon_id = sub
+    {
+      my ( $id ) = @_;
+      return "" unless defined $id;
+      $id =~ s/\r?\n$//;
 
-
-      sub prepvarnums
+      # strip path/file prefix if present
+      if ( defined( $mypath ) and ( $mypath ne "" ) and defined( $file ) and ( $file ne "" ) )
       {
-        my ( $varns_ref, $blockelts_r ) = @_;
-        my %varns = %{ $varns_ref };
-        my @blockelts = @{ $blockelts_r };
-        foreach my $key ( sort ( keys %varns ) )
-        {
-          if ( not( $key ~~ @blockelts ) )
-          {
-            $varns{$key} = 1;
-          }
-        }
-        return( %varns );
+        $id =~ s/^\Q$mypath\/$file\E//;
       }
 
-      my %varns;
-      if ( $dirfiles{starsign} eq "yes ")
+      # strip leading underscores
+      $id =~ s/^_+//;
+
+      # strip leading "file_" if present
+      if ( defined( $file ) and ( $file ne "" ) )
       {
-        %varns = prepvarnums( $dirfiles{varnumbershold}->[0], \@blockelts );
+        $id =~ s/^\Q$file\_\E//;
+      }
+      return $id;
+    };
+
+    my $compound_from_fields = sub
+    {
+      my ( $fields_r ) = @_;
+      my @f = @{ $fields_r };
+
+      # last numeric field is the safest definition across both full and short formats
+      for ( my $i = $#f; $i >= 0; $i-- )
+      {
+        if ( $is_num->( $f[$i] ) )
+        {
+          my $v = $f[$i];
+          $v =~ s/^\s+|\s+$//g;
+          return $v;
+        }
+      }
+      return ""; # should never happen for valid formats
+    };
+
+    # ---------- read and clean results (FULL format) ----------
+
+    open( my $IN, "<", $sortmixed ) or die "Cannot open $sortmixed: $!\n";
+    my @raw_full = <$IN>;
+    close $IN;
+
+    my @clean_full;
+    foreach my $ln ( @raw_full )
+    {
+      next unless defined $ln;
+      $ln =~ s/\r?\n$//;
+
+      # preserve the entire row, but canonicalize the ID field reliably
+      my @f = split( /,/, $ln, -1 );
+      next unless scalar( @f ) > 0;
+      $f[0] = $canon_id->( $f[0] );
+
+      my $rebuilt = join( ",", @f );
+      push( @clean_full, $rebuilt );
+    }
+
+    # Optional dump for review
+    if ( defined( $dowhat{dumpfiles} ) and ( $dowhat{dumpfiles} eq "y" ) )
+    {
+      my $cleanordres = $sortmixed . "_tmp_cleaned.csv";
+      open( my $CLEAN, ">", $cleanordres ) or die "Cannot write $cleanordres: $!\n";
+      foreach my $ln ( @clean_full ) { print $CLEAN $ln . "\n"; }
+      close $CLEAN;
+    }
+
+    # ---------- build blanks (IDs only) ----------
+
+    # %varns: for vars not in the block, keep one level only (as in the old pipeline)
+    my %varns = %varnums;
+    foreach my $key ( keys %varns )
+    {
+      if ( not( $key ~~ @blockelts ) )
+      {
+        $varns{$key} = 1;
+      }
+    }
+
+    my $bit = $file . "_";
+    my @bag = ( $bit );
+
+    my @box_refs = @{ Sim::OPT::toil( \@blockelts, \%varns, \@bag ) };
+    my @box      = @{ $box_refs[-1] };
+    my $integrated_r = Sim::OPT::integratebox( \@box, \%carrier, $file, \@blockelts );
+
+    my @blank_ids;
+    foreach my $el ( @{ $integrated_r } )
+    {
+      push( @blank_ids, $canon_id->( $el->[0] ) );
+    }
+    @blank_ids = uniq( @blank_ids );
+
+    if ( defined( $dowhat{dumpfiles} ) and ( $dowhat{dumpfiles} eq "y" ) )
+    {
+      my $blankfile = $sortmixed . "_tmp_blank.csv";
+      open( my $BL, ">", $blankfile ) or die "Cannot write $blankfile: $!\n";
+      foreach my $id ( @blank_ids ) { print $BL $id . "\n"; }
+      close $BL;
+    }
+
+    # ---------- normalize non-block variables (old prepfile logic) ----------
+
+    my %modhs;
+    my %torecovers;
+
+    foreach my $key ( sort keys %varnums )
+    {
+      if ( not( $key ~~ @blockelts ) )
+      {
+        $modhs{$key} = $varnums{$key};
+        $torecovers{$key} = $carrier{$key};
+      }
+    }
+
+    # Apply normalization to blanks
+    my @prep_blanks = @blank_ids;
+    if ( scalar( keys %modhs ) > 0 )
+    {
+      foreach my $id ( @prep_blanks )
+      {
+        foreach my $key ( keys %modhs )
+        {
+          $id =~ s/$key-\d+/$key-$modhs{$key}/;
+        }
+      }
+    }
+
+    # Apply normalization to results
+    my @prep_results = @clean_full;
+    if ( scalar( keys %modhs ) > 0 )
+    {
+      foreach my $ln ( @prep_results )
+      {
+        foreach my $key ( keys %modhs )
+        {
+          $ln =~ s/$key-\d+/$key-$modhs{$key}/;
+        }
+      }
+    }
+
+    if ( defined( $dowhat{dumpfiles} ) and ( $dowhat{dumpfiles} eq "y" ) )
+    {
+      my $prepblank = $sortmixed . "_tmp_prepblank.csv";
+      my $prepsort  = $sortmixed . "_tmp_prepsort.csv";
+      open( my $PB, ">", $prepblank ) or die "Cannot write $prepblank: $!\n";
+      foreach my $id ( @prep_blanks ) { print $PB $id . "\n"; }
+      close $PB;
+      open( my $PS, ">", $prepsort ) or die "Cannot write $prepsort: $!\n";
+      foreach my $ln ( @prep_results ) { print $PS $ln . "\n"; }
+      close $PS;
+    }
+
+    # ---------- join blanks + known values (FULL -> SHORT) in memory ----------
+
+    # Map: id -> compound (from FULL results; compound is last numeric field)
+    my %id2compound;
+    foreach my $ln ( @prep_results )
+    {
+      my @f = split( /,/, $ln, -1 );
+      next unless scalar( @f ) > 1;
+      my $id = $canon_id->( $f[0] );
+      my $compound = $compound_from_fields->( \@f );
+      next if ( $id eq "" );
+      next if ( $compound eq "" );
+      $id2compound{$id} = $compound;
+    }
+
+    # Interlinear input: "id" (unknown) or "id,compound" (known)
+    my @prepfile_lines;
+    foreach my $id ( @prep_blanks )
+    {
+      my $cid = $canon_id->( $id );
+      if ( exists( $id2compound{$cid} ) )
+      {
+        push( @prepfile_lines, $cid . "," . $id2compound{$cid} );
       }
       else
       {
-        %varns = prepvarnums( \%varnums, \@blockelts );
+        push( @prepfile_lines, $cid );
       }
+    }
 
-      my $cleanordres = cleanres( $sortmixed );
-	    my $blankfile = "$sortmixed" . "_tmp_blank.csv";
-	    my $bit = $file . "_";
-	    my @bag =( $bit );
-	    prepareblank( \%varns, $blankfile, \@blockelts, \@bag, $file, \%carrier ) ;
-
-
-      sub prepfile
-      {
-        my ( $blankfile, $prepblank, $cleanordres, $prepsort, $blockelts_r, $varnums_r, $carrier_r, $metafile ) = @_;
-        my @blockelts = @{ $blockelts_r };
-        my %varnums = %{ $varnums_r };
-        my %carrier = %{ $carrier_r };
-
-        my %modhs;
-        my %torecovers;
-        foreach my $el ( @blockelts )
-        {
-          foreach my $key ( sort ( keys( %varnums ) ) )
-          {
-            if ( not( $key ~~ @blockelts ) )
-            {
-              $modhs{$key} = $varnums{$key};
-              $torecovers{$key} = $carrier{$key};
-            }
-          }
-        }
-
-        open( BLANKFILE, "$blankfile" ) or die;
-        my @lines = <BLANKFILE>;
-        close BLANKFILE;
-
-        open( PREPBLANK, ">$prepblank" ) or die;
-        foreach my $line ( @lines )
-        {
-          foreach my $key ( keys %modhs )
-          {
-            $line =~ s/$key-\d+/$key-$modhs{$key}/ ;
-          }
-          print PREPBLANK $line;
-        }
-        close PREPBLANK;
-
-        open( CLEANORDRES, "$cleanordres" ) or die;
-        my @lines = <CLEANORDRES>;
-        close CLEANORDRES;
-
-        open( PREPSORT, ">$prepsort" ) or die;
-        foreach my $line ( @lines )
-        {
-          foreach my $key ( keys %modhs )
-          {
-            $line =~ s/$key-\d+/$key-$modhs{$key}/ ;
-          }
-          print PREPSORT $line;
-        }
-        close PREPSORT;
-
-        return( %torecovers );
-      }
-
-      my $prepblank = $sortmixed . "_tmp_prepblank.csv";
-      my $prepsort = $sortmixed . "_tmp_prepsort.csv";
-      my %torecovers = prepfile( $blankfile, $prepblank, $cleanordres, $prepsort, \@blockelts, \%varnums, \%carrier, $metafile );
-
-	    sub jointwo
-	    {
-	      my ( $prepsort, $prepblank, $prepfile ) = @_;
-	      open( PREPBLANK, $prepblank ) or die;
-	      my @blanklines = <PREPBLANK>;
-        @blanklines = uniq( @blanklines );
-	      close PREPBLANK;
-
-	      open( PREPSORT, $prepsort ) or die;
-	      my @ordlines = <PREPSORT>;
-        @ordlines = uniq( @ordlines );
-	      close PREPSORT;
-
-	      open( PREPFILE, ">$prepfile" ) or die;
-
-	      my @newarr;
-	      foreach my $blankline ( @blanklines )
-	      {
-	        chomp $blankline;
-	        my $signal = "n";
-	        foreach my $ordline ( @ordlines )
-	        {
-	          chomp $ordline;
-	          my @row = split( "," , $ordline );
-	          if ( ( $row[0] =~ /$blankline/ ) or ( /$blankline/ =~ $row[0] ) )
-	          {
-	            say PREPFILE "$blankline" . "," . "$row[1]" ;
-	            $signal = "y";
-	          }
-	        }
-	        if ( $signal eq "n" )
-	        {
-	          say PREPFILE $blankline ;
-	        }
-	      }
-	      close PREPFILE;
-	    }
-
+    if ( defined( $dowhat{dumpfiles} ) and ( $dowhat{dumpfiles} eq "y" ) )
+    {
       my $prepfile = $sortmixed . "_tmp_prepfile.csv";
-	    jointwo( $prepsort, $prepblank, $prepfile ) ;
+      open( my $PF, ">", $prepfile ) or die "Cannot write $prepfile: $!\n";
+      foreach my $ln ( @prepfile_lines ) { print $PF $ln . "\n"; }
+      close $PF;
+    }
 
-      my $rawmetafile = $metafile . "_tmp_raw.csv";
-	    #say $tee "ABOUT TO LAUNCH INTERLINEAR";
+    # ---------- call Interlinear (arrayref input; you already enabled this) ----------
 
-      my @prepwelds = @{ $dowhat{prewelds} };
-      my @parswelds = @{ $dowhat{parswelds} };
-      my @weldsprepared;
+    my $rawmetafile = $metafile . "_tmp_raw.csv";
+    Sim::OPT::Interlinear::interlinear( \@prepfile_lines, $confinterlinear, $rawmetafile, \@blockelts, $tofile, $countblock, $dowhat_r, $dirfiles_r );
 
-      if ( scalar( @prepwelds ) > 0 )
+    # ---------- restore non-block variables & write meta outputs ----------
+
+    open( my $RM, "<", $rawmetafile ) or die "Cannot open $rawmetafile: $!\n";
+    my @rawlines = <$RM>;
+    close $RM;
+
+    my @metas;
+    foreach my $ln ( @rawlines )
+    {
+      next unless defined $ln;
+      $ln =~ s/\r?\n$//;
+
+      foreach my $key ( keys %torecovers )
       {
-        my $c = 0;
-        foreach my $weld ( @prepwelds )
-        {
-          unless ( -e $weld )
-          {
-            die;
-          }
-          my $prepared = $weld . ".weldprep.csv";
-          push ( @weldsprepared, $prepared );
-
-          $c++;
-        }
+        $ln =~ s/$key-\d+/$key-$torecovers{$key}/;
       }
+      push( @metas, $ln );
+    }
 
-	    Sim::OPT::Interlinear::interlinear( $prepfile, $confinterlinear, $rawmetafile, \@blockelts, $tofile, $countblock );
+    @metas = uniq( @metas );
 
-      open( RAWMETAFILE, "$rawmetafile" ) or die;
-      my @rawlines = <RAWMETAFILE>;
-      close RAWMETAFILE;
+    open( my $MF, ">", $metafile ) or die "Cannot write $metafile: $!\n";
+    foreach my $ln ( @metas ) { print $MF $ln . "\n"; }
+    close $MF;
 
-      open( METAFILE, ">$metafile" ) or die;
-      foreach my $line ( @rawlines )
-      {
-        foreach my $key ( keys %torecovers )
-        {
-          $line =~ s/$key-\d+/$key-$torecovers{$key}/ ;
-        }
-        print METAFILE $line;
-      }
-      close METAFILE;
-
-	    open( METAFILE, "$metafile" ) or die "THERE IS NO METAFILE $metafile!\n";
-	    my @totlines = <METAFILE>;
-      @totlines = uniq( @totlines );
-	    close METAFILE;
-
-	    if ( ( $direction eq "<" ) or ( $starorder eq "<" ) )
-	    {
-	      @sorted = sort { (split( ',', $a))[ 1 ] <=> ( split( ',', $b))[ 1 ] } @totlines;
-	    }
-	    elsif ( ( $direction eq ">" ) or ( $starorder eq ">" ) )
-	    {
-	      @sorted = sort { (split( ',', $b))[ 1 ] <=> ( split( ',', $a))[ 1 ] } @totlines;
-	    }
-
-      @sorted = uniq( @sorted );
-
-	    open( ORDMETA,  ">>$ordmeta" ) or die;
-	    foreach my $ln ( @sorted )
-	    {
-	      print ORDMETA $ln;
-	    }
-	    close ORDMETA;
-		}
-	}  # END SUB metamodel
+    # Sort and write ORDMETA (overwrite, not append)
+    my @sorted;
+    if ( ( defined( $direction ) and ( $direction eq "<" ) ) or ( defined( $starorder ) and ( $starorder eq "<" ) ) )
+    {
+      @sorted = sort { (split( /,/, $a, -1 ))[1] <=> ( split( /,/, $b, -1 ))[1] } @metas;
+    }
+    else
+    {
+      @sorted = sort { (split( /,/, $b, -1 ))[1] <=> ( split( /,/, $a, -1 ))[1] } @metas;
+    }
 
+    open( my $OM, ">", $ordmeta ) or die "Cannot write $ordmeta: $!\n";
+    foreach my $ln ( @sorted ) { print $OM $ln . "\n"; }
+    close $OM;
+  }  # END SUB metamodel
 
   sub takeoptima
   {
