@@ -23,7 +23,7 @@ use Sim::OPT::Sim;
 use Sim::OPT::Descend;
 use Sim::OPT::Takechance;
 use Sim::OPT::Interlinear;
-eval { use Sim::OPTcue::Patternsearch; 1 };
+use Sim::OPT::Stats;
 eval { use Sim::OPTcue; 1 };
 
 
